# Generated by Django 3.2.13 on 2022-05-20 21:11

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('djangocms_versioning', '0015_version_modified'),
    ]

    operations = [
        migrations.AlterField(
            model_name='version',
            name='content_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='cms_versions', to='contenttypes.contenttype'),
        ),
    ]
