{% extends versioning_fallback_change_form_template|default:"admin/change_form.html" %}
{% load static %}

{% block extrahead %}
    {{ block.super }}
    <script src="{% static "djangocms_versioning/js/object-tools.js" %}"></script>
{% endblock %}

{% block extrastyle %}
    {{ block.super }}
    <link rel="stylesheet" href="{% static "djangocms_versioning/css/object-tools.css" %}">
{% endblock %}

{% block object-tools-items %}
    {% include "admin/djangocms_versioning/versioning_buttons.html" %}
    {{ block.super }}
{% endblock %}

