# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-24 12:42
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("djangocms_versioning", "0001_initial")]

    operations = [migrations.DeleteModel(name="Campaign")]
