{% extends "admin/change_form.html" %}
{% load i18n admin_urls static admin_list %}

{% block bodyclass %}{{ block.super }} app-{{ opts.app_label }} model-{{ opts.model_name }} change-list{% endblock %}

{% if not is_popup %}
{% block breadcrumbs %}
<div class="breadcrumbs">
<a href="{% url 'admin:index' %}">{% translate 'Home' %}</a>
&rsaquo; <a href="{% url 'admin:app_list' app_label=opts.app_label %}">{{ opts.app_config.verbose_name }}</a>
&rsaquo; {{ opts.verbose_name_plural|capfirst }}
</div>
{% endblock %}
{% endif %}

{% block extrastyle %}{{ block.super }}{{ form.media }}{% endblock extrastyle %}
{% block coltype %}flex{% endblock %}

{% block content %}
  <div id="content-main">
    {% block object-tools %}
        <ul class="object-tools">
          {% block object-tools-items %}
            {% if has_add_permission %}
            <li>
              {% url opts|admin_urlname:'add' as add_url %}
              <a href="{% add_preserved_filters add_url is_popup to_field %}" class="addlink">
                {% blocktrans with opts.verbose_name as name %}Add {{ name }}{% endblocktrans %}
              </a>
            </li>
            {% endif %}
          {% endblock %}
        </ul>
    {% endblock %}
    <div class="module" id="changelist">
      <form action="{% url opts|admin_urlname:"changelist" %}">
        <fieldset class="module aligned">
          <div class="form-row">
            <div class="flex-container">
              {{ form }}
            </div>
          </div>
        </fieldset>
        <button type="submit">{% translate "Submit" %}</button>
        </form>
      </form>
    </div>
  </div>
{% endblock %}
